<?php
/**
 * Plugin Publication multi-sites avec filtrage par rubrique
 * Copyright (c) Christophe IMBERTI
 * Licence Creative commons by-nc-sa
 */


/**
 * Chemin complet du dossier tmp propre a chaque site (slash de fin compris)
 */
function cims_dir_tmp() {
        $ci_dir_racine_de_tmp = cims_racine_dossier(_NOM_TEMPORAIRES_INACCESSIBLES);
    
	$ci_dir_tmp = "tmp/tmp" . cims_site_en_cours();
	$ci_path = $ci_dir_racine_de_tmp . $ci_dir_tmp;
	if (!@is_dir($ci_path)){
		cims_creer_dir_tmp($ci_path,$ci_dir_racine_de_tmp);
        }
        
	return $ci_dir_tmp."/";
}

function cims_racine_dossier($dossier=''){
        $ci_dir_racine_dossier = '';
        
        // Possibilite d'utiliser la fonction facultative cims_racine_dossier_options()
        // qui devra se trouver dans le fichier config/mes_options.php
        // AVANT include_spip('cims_options')
        // Elle doit retourner la racine du dossier tmp/ (ou autre)
        if($dossier AND function_exists('cims_racine_dossier_options')) {
            $ci_dir_racine_dossier = cims_racine_dossier_options($dossier);
        }
        
        if (!$ci_dir_racine_dossier){
            $ci_dir_racine_dossier = _DIR_RACINE;
        } else {
            if (substr($ci_dir_racine_dossier,-1)!='/'){
                $ci_dir_racine_dossier .= '/';
            }
        }
    
        return $ci_dir_racine_dossier;
}

/**
 * Creer le dossier tmp propre au site
 */
function cims_creer_dir_tmp($ci_dir_tmp,$ci_dir_racine_de_tmp='') {
	// Creer le dossier
	@mkdir($ci_dir_tmp, 0777);
	@mkdir($ci_dir_tmp."/cache", 0777);

	// copier certains fichiers pour eviter d'activer les plugins communs aux sites
	// y compris la compatibilite avec SPIP 2.1
	$f_tmp = array('meta_cache.php','plugin_xml.cache','plugin_xml_cache.gz','charger_pipelines.php','charger_plugins_fonctions.php','charger_plugins_options.php','verifier_plugins.txt');
	foreach($f_tmp as $f) {
		if (file_exists($ci_dir_racine_de_tmp . "tmp/" . $f)) {
			@copy($ci_dir_racine_de_tmp . "tmp/" . $f, $ci_dir_tmp ."/" . $f);
		}
	}

	$f_tmp = array('charger_plugins_chemins.php','charger_pipelines.php','charger_plugins_fonctions.php','charger_plugins_options.php','verifier_plugins.txt');
	foreach($f_tmp as $f) {
		if (file_exists($ci_dir_racine_de_tmp . "tmp/cache/" . $f)) {
			@copy($ci_dir_racine_de_tmp . "tmp/cache/" . $f, $ci_dir_tmp ."/cache/" . $f);
		}
	}

	return true;
}


/**
 * Determiner le site en cours en fonction du host
 */
function cims_site_en_cours() {
	static $ci_site;
	
	if ($ci_site===NULL) {
		$ci_site='';
		
		// Le multi sites est-il actif ?
		// au passage cela indique que le fichier de parametrage est charge
		if (defined('_CIMS_ACTIF')) {
                        $tableau_config = cims_lire_config();

			// determiner le HOST
			$ci_host = cims_url_host();
		
			// site par defaut
			if (isset($tableau_config['cisite']))
				$ci_site = $tableau_config['cisite'];
		
				
			// determiner le site en cours en fonction du host
			if (isset($tableau_config['ciadressedessites'])) {
				if (is_array($tableau_config['ciadressedessites'])) {
                                        foreach ($tableau_config['ciadressedessites'] as $host => $site) {
						if ($host==$ci_host) {
							$ci_site=$site;
							break;
						}
					}
				}
			}
                        
                        // Possibilite d'utiliser la fonction facultative cims_siteencours_options()
                        // qui devra se trouver (imperativement) en dernier dans le fichier config/mes_options.php
                        // Remarque : un pipeline ne peut pas etre utilise car le present fichier 
                        // est appele par mes_options.php (qui ne permet pas d'utiliser un pipeline)
                        if(function_exists('cims_siteencours_options')) {
                            $ci_site = cims_siteencours_options($ci_site);
                        }                  
                        
		}

	}

	return $ci_site;
}


/**
 * Determiner le HOST
 */
function cims_url_host() {
	
	$ci_host = "";

	if (defined('_CIMS_ACTIF')) {
                $tableau_config = cims_lire_config();
            
		// ordre de recherche par defaut (celui de phpCAS)
		$cimshostordre = array('HTTP_X_FORWARDED_SERVER','SERVER_NAME','HTTP_HOST');
			
		// ordre de recherche defini dans le fichier de parametrage config/_config_multisites.php
		if (isset($tableau_config['cimshostordre'])) {
			if (is_array($tableau_config['cimshostordre'])) {
				$cimshostordre = $tableau_config['cimshostordre'];
			}
		}
		
		foreach ($cimshostordre as $valeur) {
			if (isset($_SERVER[$valeur])) {
				if ($_SERVER[$valeur]) {
					$ci_host = $_SERVER[$valeur];
					break;
				}
			}
		}
	}

	return $ci_host;
}


/**
 * Lire le fichier de parametrage config/_config_multisites.php
 */
function cims_lire_fichier_parametrage() {
	static $ci_lecture;
	
	if ($ci_lecture===NULL) {
		$ci_lecture = true;		
		$f = cims_racine_dossier(_NOM_PERMANENTS_INACCESSIBLES) . _NOM_PERMANENTS_INACCESSIBLES . '_config_multisites.php';

		if (@file_exists($f))
			include_once($f);
	}

    return true;
}

/**
 * Lecture des parametres de configuration du plugin
 * S'il existe, le parametrage par fichier est prioritaire
 *
 * @param : aucun
 * @return : tableau de config
 */
function cims_lire_config() {
        static $done = false;
        static $config = array();

        if (!$done){

                $f = cims_racine_dossier(_NOM_PERMANENTS_INACCESSIBLES) . _NOM_PERMANENTS_INACCESSIBLES . '_config_multisites.php';

                if (@file_exists($f)) {
                        // parametrage par fichier
                        include_once($f);
                }
                
                // memoriser
                if (isset($GLOBALS['ciconfig'])){
                    $config = $GLOBALS['ciconfig'];
                }                
                $done = true;		
        }

        return $config;
}

/**
 * Numero du site en cours
 * Cette fonction est utilisee par ciparam
 */
function cims_num_site($arg='') {
	static $ci_num_site;
	
	if (!$ci_num_site) {
		$ci_num_site=0;
		
		if (defined('_CIMS_ACTIF')) {
                        $tableau_config = cims_lire_config();
                    
			$ci_site = cims_site_en_cours();
	
			// determiner le rang du site en cours
			// dans le tableau de parametrage
                        if ($ci_site){
                                if (isset($tableau_config['cinumlogodessites'][$ci_site])){
                                    $ci_num_site = 1 + intval($tableau_config['cinumlogodessites'][$ci_site]);

                                } elseif (isset($tableau_config['cilistedessites'])) {
                                        if (is_array($tableau_config['cilistedessites'])) {
                                                foreach ($tableau_config['cilistedessites'] as $site) {
                                                        $ci_num_site++;
                                                        if ($site==$ci_site)
                                                                break;
                                                }
                                        }
                                }
                        }
		}
	}

	return $ci_num_site;
}


/**
 * Image de bandeau du site en cours
 * Cette fonction est utilisee par ciparam
 */
function cims_image_bandeau() {
    $return = '';
    
    if ($GLOBALS['spip_version_branche']>=4){
        $info_logo = quete_logo_objet(0, 'site', 'on');
        if (isset($info_logo['chemin'])){
            $return = $info_logo['chemin'];
        }
    } else {	
        if (defined('_CIMS_ACTIF'))
                // tiret indispensable apres siteon
                $nom = "siteon-".cims_num_site();
        else
                $nom = "siteon0";

        $formats_logos =  array ('gif', 'jpg', 'png');
        foreach ($formats_logos as $format) {
                if (@file_exists($d = (_DIR_LOGOS . $nom . '.' . $format))) {
                        $return = $d;
                }
        }
    }
    
    return $return;
}


/**
 * Le parametrage du multi sites est-il actif dans le fichier de parametrage ?
 */
function cims_parametrage_actif() {
	
	$actif = false;
	
	// lire le fichier de parametrage config/_config_multisites.php
	$tableau_config = cims_lire_config();
	
	// Le multi sites est-il actif dans le fichier de parametrage ?
	if (isset($tableau_config['cimultisites'])) {
		if ($tableau_config['cimultisites']=="oui")
			$actif = true;
	}

	return $actif;
}


/**
 * Le filtrage par rubrique est-il actif pour ce site dans le fichier de parametrage ?
 */
function cims_filtrage_actif() {
	
	$actif = false;
		
	// Le multi sites est-il actif
	if (defined('_CIMS_ACTIF')) {
		$actif = true;
                $tableau_config = cims_lire_config();

		// le filtrage par rubrique est-il desactive pour le site en cours ?
		if (isset($tableau_config['cimsfiltragedesactive'])) {
			if (is_array($tableau_config['cimsfiltragedesactive'])) {
				if (in_array(cims_site_en_cours(),$tableau_config['cimsfiltragedesactive'])) {
					$actif = false;
				}
			}
		}
	}

	return $actif;
}

?>