<?php

/***************************************************************************\
 *  SPIP, Systeme de publication pour l'internet                           *
 *                                                                         *
 *  Copyright (c) 2001-2010                                                *
 *  Arnaud Martin, Antoine Pitrou, Philippe Riviere, Emmanuel Saint-James  *
 *                                                                         *
 *  Ce programme est un logiciel libre distribue sous licence GNU/GPL.     *
 *  Pour plus de details voir le fichier COPYING.txt ou l'aide en ligne.   *
\***************************************************************************/

if (!defined("_ECRIRE_INC_VERSION")) return;

// Les parametres generaux du site sont dans une table SQL;
// Recopie dans le tableau PHP global meta, car on en a souvent besoin

// duree maximale du cache. Le double pour l'antidater
define('_META_CACHE_TIME', 1<<24);

//----- Debut ajout CI -----
// les plugins ne sont pas encore charges
include_spip('cims_options');
//----- Fin ajout CI -----


// http://doc.spip.org/@inc_meta_dist
function inc_meta_dist($table='meta')
{
	// Lire les meta, en cache si present, valide et lisible
	// en cas d'install ne pas faire confiance au meta_cache eventuel
	$cache = cache_meta($table);
        
        $meta = '';

	if ((_request('exec')!=='install' OR !test_espace_prive())
	AND $new = jeune_fichier($cache, _META_CACHE_TIME)
	AND lire_fichier_securise($cache, $meta)
	AND $meta = @unserialize($meta))
		$GLOBALS[$table] = $meta;

	if (isset($GLOBALS[$table]['touch']) 
	AND ($GLOBALS[$table]['touch']<time()-_META_CACHE_TIME))
		$GLOBALS[$table] = array();
	// sinon lire en base
	if (!$GLOBALS[$table]) $new = !lire_metas($table);

	// renouveller l'alea general si trop vieux ou sur demande explicite
	if ((test_espace_prive() || isset($_GET['renouvelle_alea']))
	AND $GLOBALS[$table]
	AND (time() > _RENOUVELLE_ALEA + $GLOBALS['meta']['alea_ephemere_date'])) {
		// si on n'a pas l'acces en ecriture sur le cache,
		// ne pas renouveller l'alea sinon le cache devient faux
		if (supprimer_fichier($cache)) {
			include_spip('inc/acces');
			renouvelle_alea();
			$new = false; 
		} else spip_log("impossible d'ecrire dans " . $cache);
	}
	// et refaire le cache si on a du lire en base
	if (!$new) touch_meta(false, $table);
}


// fonctions aussi appelees a l'install ==> spip_query en premiere requete 
// pour eviter l'erreur fatale (serveur non encore configure)

// http://doc.spip.org/@lire_metas
function lire_metas($table='meta') {

//----- Debut ajout CI -----
	$cims_table = cims_table_meta();
	
//	if ($result = spip_query("SELECT nom,valeur FROM spip_$table")) {
	if ($result = sql_select("nom,valeur","spip_".$cims_table)) {
//----- Fin ajout CI -----
		
		include_spip('base/abstract_sql');
		$GLOBALS[$table] = array();
		while ($row = sql_fetch($result))
			$GLOBALS[$table][$row['nom']] = $row['valeur'];
			
//----- Debut ajout CI -----
		// Traiter les exceptions � prendre dans spip_meta
		if ($table=='meta') {
			$exceptions = cims_meta_exceptions();
			if ($result = sql_select("nom,valeur","spip_meta")) {
				while ($row = sql_fetch($result)) {
					if (in_array($row['nom'],$exceptions))
						$GLOBALS['meta'][$row['nom']] = $row['valeur'];
				}
			}
		}
//----- Fin ajout CI -----
			
			
		if (!$GLOBALS[$table]['charset']
		  OR $GLOBALS[$table]['charset']=='_DEFAULT_CHARSET' // hum, correction d'un bug ayant abime quelques install
		)
			ecrire_meta('charset', _DEFAULT_CHARSET, NULL, $table);
	}
	return $GLOBALS[$table];
}


// Mettre en cache la liste des meta, sauf les valeurs sensibles 
// pour qu'elles ne soient pas visibiles dans un fichier.souvent en 777
// http://doc.spip.org/@touch_meta
function touch_meta($antidate= false, $table='meta'){
	$file = cache_meta($table);
	if (!$antidate OR !@touch($file, $antidate)) {
		$r = $GLOBALS[$table];
		unset($r['alea_ephemere']);
		unset($r['alea_ephemere_ancien']);
		// le secret du site est utilise pour encoder les contextes ajax que l'on considere fiables
		// mais le sortir deu cache meta implique une requete sql des qu'on a un form dynamique
		// meme si son squelette est en cache
		//unset($r['secret_du_site']);
		if ($antidate) $r['touch']= $antidate;
		ecrire_fichier_securise($file, serialize($r));
	}
}


// http://doc.spip.org/@effacer_meta
function effacer_meta($nom, $table='meta') {
	// section critique sur le cache:
	// l'invalider avant et apres la MAJ de la BD
	// c'est un peu moins bien qu'un vrai verrou mais ca suffira
	// et utiliser une statique pour eviter des acces disques a repetition
	static $touch = array();
	
//----- Debut ajout CI -----
	$cims_table = cims_table_meta();
//----- Fin ajout CI -----
	
	$antidate = time() - (_META_CACHE_TIME<<4);
	if (!isset($touch[$table])) {touch_meta($antidate, $table);}
	
//----- Debut ajout CI -----
//	sql_delete('spip_' . $table, "nom='$nom'");
	sql_delete('spip_' . $cims_table, "nom='$nom'");

	// Traiter les exceptions � prendre dans spip_meta
	if ($table=='meta') {
		$exceptions = cims_meta_exceptions();
		if (in_array($nom,$exceptions))
			sql_delete('spip_meta', "nom='$nom'");
	}		
//----- Fin ajout CI -----

	unset($GLOBALS[$table][$nom]);	
	if (!isset($touch[$table])) {touch_meta($antidate, $table); $touch[$table] = false;}
}


// http://doc.spip.org/@ecrire_meta
function ecrire_meta($nom, $valeur, $importable = NULL, $table='meta') {

	static $touch = array();
	if (!$nom) return;
	
	include_spip('base/abstract_sql');
//----- Debut ajout CI -----
//	$res = sql_select("*",'spip_' . $table,"nom=" . sql_quote($nom),'','','','','','continue');
	$cims_table = cims_table_meta();
	if ($table=='meta')
		$exceptions = cims_meta_exceptions();

	$res = sql_select("*",'spip_' . $cims_table,"nom=" . sql_quote($nom),'','','','','','continue');
//----- Fin ajout CI -----

	// table pas encore installee, travailler en php seulement
	if (!$res) {
		$GLOBALS[$table][$nom] = $valeur;
		return;
	}
	$res = sql_fetch($res);
	// ne pas invalider le cache si affectation a l'identique
	// (tant pis si impt aurait du changer)
	if ($res AND $valeur == $res['valeur'] AND $GLOBALS[$table][$nom] == $valeur) return;
	$GLOBALS[$table][$nom] = $valeur;
	// cf effacer pour comprendre le double touch
	$antidate = time() - (_META_CACHE_TIME<<1);
	
	if (!isset($touch[$table])) {touch_meta($antidate, $table);}

	$r = array('nom' => $nom, 'valeur' => $valeur);
	// Gaffe aux tables sans impt (vieilles versions de SPIP notamment)
	if ($importable AND isset($res['impt'])) $r['impt'] = $importable;
	
//----- Debut ajout CI -----
/*
	if ($res) {
		sql_updateq('spip_' . $table, $r,"nom=" . sql_quote($nom));
	} else {
		sql_insertq('spip_' . $table, $r);
	}
*/
	if ($res) {
		sql_updateq('spip_' . $cims_table, $r,"nom=" . sql_quote($nom));
		// Traiter les exceptions � prendre dans spip_meta
		if ($table=='meta') {
			if (in_array($nom,$exceptions))
				sql_updateq('spip_meta', $r,"nom=" . sql_quote($nom));
		}
	} else {
		sql_insertq('spip_' . $cims_table, $r);
		// Traiter les exceptions � prendre dans spip_meta
		if ($table=='meta') {
			if (in_array($nom,$exceptions))
				sql_insertq('spip_meta', $r);
		}
	}
	
	// si dernier modif supprimer les fichiers meta_cache.php de chaque autre site
//	if (strlen($nom)>=14 AND substr($nom,0,14)=='derniere_modif'){
	if ($nom=='derniere_modif'){
                $tableau_config = cims_lire_config();
		$ci_site_en_cours = cims_site_en_cours();
                $ci_chemin_tmp = cims_racine_dossier(_NOM_TEMPORAIRES_INACCESSIBLES) . _NOM_TEMPORAIRES_INACCESSIBLES;
                
		if (isset($tableau_config['cilistedessites'])) {
			if (is_array($tableau_config['cilistedessites'])) {
				foreach ($tableau_config['cilistedessites'] as $site) {
					if ($site!=$ci_site_en_cours){
						supprimer_fichier($ci_chemin_tmp."tmp".$site.'/meta_cache.php');
                                        }
				}
			}
		}
	}
//----- Fin ajout CI -----
	if (!isset($touch[$table])) {touch_meta($antidate, $table); $touch[$table] = false;}
}


function cache_meta($table='meta')
{
	return ($table=='meta') ? _FILE_META : (_DIR_CACHE . $table . '.php');
}


//----- Debut ajout CI -----


// Nom de la table spip_meta... du site en cours (sans le prefixe "spip_")
function cims_table_meta($table='meta') {
	include_spip('base/abstract_sql');
	
	$exist = "";
	$ci_site_en_cours = cims_site_en_cours();
	$table_site = "cims_".$table.cims_site_en_cours();

	$res = sql_showbase("spip_".$table_site);
	if ($res) $exist = sql_fetch($res);

	if ($exist) {
		$table = $table_site;
	} else {
		// creer la table et pre remplir a partir de spip_meta
		cims_creer_table_meta($table_site,$table);

		$res2 = sql_showbase("spip_".$table_site);
		if ($res2) $exist2 = sql_fetch($res2);
		if ($exist2)
			$table = $table_site;
	}

	return $table;
}


// Creer la table spip_meta... du site en cours et pre remplir le cas echeant
function cims_creer_table_meta($table_site,$table) {
	include_spip('base/abstract_sql');
	include_spip('base/create');	

	$table_site = 'spip_'.$table_site;
	$table = 'spip_'.$table;
	
	$trouver_table = charger_fonction('trouver_table','base');
	if ($desc = $trouver_table($table)) {
		// Creer la table
		creer_ou_upgrader_table($table_site,$desc,true,false,'');

		// Pre remplir la table
		if ($result = sql_select("*",$table)) {
			while ($row = sql_fetch($result))
				sql_insertq($table_site, $row);
		}
		
	}

	return true;
}


// Exceptions � prendre dans spip_meta
function cims_meta_exceptions() {

	return array('version_installee',
		'nouvelle_install',
		'charset_sql_connexion',
		'charset_sql_base',
		'charset_collation_sql_base',
		'mysql_rappel_nom_base',
		'max_taille_vignettes_test',
		'date_statistiques',
		'date_popularites',
		'date_calcul_rubriques',
		'alea_ephemere',
		'alea_ephemere_ancien',
		'alea_ephemere_date', 
		'admin',
		'secret_du_site',
		'upgrade',
		'admin_repair',
		'convert_sql_utf8',
		'convert_utf8',
		'delete_all',
		'delete_statistiques',
		'import_all',
		'derniere_modif',
		'derniere_modif_article',
		'derniere_modif_syndic',
		'derniere_modif_rubrique'
		);
}
//----- Fin ajout CI -----

?>