<?php
/**
 * Plugin CIWIDGET
 * Copyright (c) Christophe IMBERTI
 * Licence Creative commons by-nc-sa
 */

if (!defined("_ECRIRE_INC_VERSION")) return;

function ciwidget_lire_meta() {
    static $ciwidget_config = array();
    $ciwidget_config_tmp = array();

    if (!$ciwidget_config) {
        if (isset($GLOBALS['meta']['ciwidget'])){
            $metas = @unserialize($GLOBALS['meta']['ciwidget']);
        } else {
            $metas = array();
        }
        
        $ciwidget_parametrage_par_fichier = ciwidget_contenu_du_parametrage_par_fichier();
        $ciwidget_parametrage_rgpd_par_fichier = ciwidget_contenu_du_parametrage_rgpd_par_fichier();
            
        if ($ciwidget_parametrage_par_fichier) {
            // le parametrage par fichier est prioritaire
            $ciwidget_config_tmp = $ciwidget_parametrage_par_fichier;
            
            include_spip('inc/ciwidget_commun');    
            $types_widget = ciwidget_types_widget();
            
            foreach ($types_widget AS $cle=>$type_widget) {
                $type = '';
                if (isset($type_widget['id'])) {
                    $type = $type_widget['id'];
                }
                
                // Gerer les exceptions non imposables
                if ($type AND isset($type_widget['non_imposable']) AND $type_widget['non_imposable']=='oui'){
                    $done = false;
                    if (isset($GLOBALS['meta']['ciwidget']) AND $GLOBALS['meta']['ciwidget']){
                        if (isset($metas[$type]) AND $metas[$type]){
                            $ciwidget_config_tmp[$type] = $metas[$type];
                            $done = true;
                        }
                    }
                    
                    if (!$done AND isset($ciwidget_config_tmp[$type])){
                        unset($ciwidget_config_tmp[$type]);
                    }
                } elseif ($type) {
                    // si pas de config imposee, dans le fichier, pour un type de widget
                    // prendre celle dans metas
                    if (!isset($ciwidget_config_tmp[$type])){
                        if (isset($metas[$type]) AND $metas[$type]){
                            $ciwidget_config_tmp[$type] = $metas[$type];
                        }
                    }
                }
            }

            // cas de statut
            $type_statut = 'statut';
            if (isset($metas[$type_statut]) AND $metas[$type_statut]){
                $ciwidget_config_tmp[$type_statut] = $metas[$type_statut];
            }
            
            // cas du RGPD
            $type_statut = 'rgpd';
            if (isset($metas[$type_statut]) AND $metas[$type_statut]){
                $ciwidget_config_tmp[$type_statut] = $metas[$type_statut];
            }
            
        } elseif (isset($GLOBALS['meta']['ciwidget']) AND $GLOBALS['meta']['ciwidget']) {
            $ciwidget_config_tmp = $metas;
        }
        
        // cas du RGPD
        if ($ciwidget_parametrage_rgpd_par_fichier){
            $ciwidget_config_tmp['rgpd'] = $ciwidget_parametrage_rgpd_par_fichier;
        }
        
        
        $ciwidget_config = $ciwidget_config_tmp;
    }
    
    return $ciwidget_config;
}

function ciwidget_lire_meta_widget($widget) {
    $return_config = array();
    $return_defaut = array();
    
    $ciwidget_config = ciwidget_lire_meta();
    if (isset($ciwidget_config[$widget])) {
        $return_config = $ciwidget_config[$widget];
    }
    
    // sinon les valeurs par défaut
    $ciwidget_defaut = ciwidget_valeurs_par_defaut();
    if (isset($ciwidget_defaut[$widget])) {
        $return_defaut = $ciwidget_defaut[$widget];
    }
    
    return array_merge($return_defaut,$return_config);
}

function ciwidget_lire_meta_cle($widget,$cle) {
    $ciwidget_config_widget = array();
    $return = '';
    
    if (empty($widget) OR empty($cle)){
        return '';
    }

    // cas particulier du statut
    if ($cle=='statut'){
        $ciwidget_config_statut = ciwidget_lire_meta_widget('statut');
        if (isset($ciwidget_config_statut['statut_'.$widget])) {
            $return = $ciwidget_config_statut['statut_'.$widget];
            if (!$return){
                $return = 'active';
            }
        } else {
            $return = 'active';
        }
        
    // cas particulier du rgpd
    } elseif ($cle=='rgpd'){
        $ciwidget_config_rgpd = ciwidget_lire_meta_widget('rgpd');
        if (isset($ciwidget_config_rgpd['rgpd_'.$widget])) {
            $return = $ciwidget_config_rgpd['rgpd_'.$widget];
            if (!$return){
                $return = 'non';
            }
        } else {
            $return = 'non';
        }
        
    } else {
        $ciwidget_config_widget = ciwidget_lire_meta_widget($widget);
        if (isset($ciwidget_config_widget[$cle])) {
            include_spip('inc/ciwidget_verifier');
            if (ctype_alnum(str_replace('_','',$cle)) AND function_exists($f = 'ciwidget_verifier_' . $cle)) {
                if ($f($ciwidget_config_widget[$cle])){
                    $return = $ciwidget_config_widget[$cle];
                }
            } else {
                $return = $ciwidget_config_widget[$cle];
            }
        }
    }
    
    return $return;
}

function ciwidget_ecrire_meta($widget,$champs) {
    $ciwidget_config = array();

    if ($widget AND $champs AND is_array($champs)){
        if (isset($GLOBALS['meta']['ciwidget'])) {
            $ciwidget_config = @unserialize($GLOBALS['meta']['ciwidget']);
        }

        $ciwidget_config[$widget] = $champs;    
        include_spip('inc/meta');
        ecrire_meta('ciwidget', @serialize($ciwidget_config));
    }
}

function ciwidget_supprime_meta_widget($widget) {
    $ciwidget_config = array();

    if ($widget){
        if (isset($GLOBALS['meta']['ciwidget'])) {
            $ciwidget_config = @unserialize($GLOBALS['meta']['ciwidget']);
        }

        if (isset($ciwidget_config[$widget])){
            unset($ciwidget_config[$widget]);  
        }
        include_spip('inc/meta');
        ecrire_meta('ciwidget', @serialize($ciwidget_config));
    }
}

function ciwidget_valeurs_par_defaut(){
    include_spip('inc/ciwidget_commun');
    
    $return = array(
        'commun' => array(
                'largeur_min'=>'100',
                'largeur_max'=>'1024',
                'hauteur_min'=>'100',
                'hauteur_max'=>'768',
                'debit_video_max'=>'',
                'video_num_skin'=>'1',
                'mp4'=>'oui',
                'webm'=>'oui',
                'ogv'=>'oui',
                'mp3'=>'oui',
                'wav'=>'oui',
                'ogg'=>'oui'
                ),
        'video' => array(
                'largeur_defaut'=>'480',
                'hauteur_defaut'=>'270'
                ),
        'videodistante' => array(
                'largeur_defaut'=>'480',
                'hauteur_defaut'=>'270'
                ),
        'streamhls' => array(
                'largeur_defaut'=>'640',
                'hauteur_defaut'=>'480'
                )
        );
    
    $types_widget = ciwidget_types_widget();
    $tableau_statut = array();
    $tableau_rgpd = array(); 
    
    foreach ($types_widget AS $cle=>$type_widget) {
        if (isset($type_widget['id']) AND isset($type_widget['valeurs_par_defaut'])){
            $return[$type_widget['id']] = $type_widget['valeurs_par_defaut'];
        }
        $tableau_statut['statut_'.$type_widget['id']] = 'active';
        $tableau_rgpd['rgpd_'.$type_widget['id']] = 'non';
        
        // valeur de base par defaut si valeur par defaut non renseignee
        if (!isset($return[$type_widget['id']]['statut'])){
            $return[$type_widget['id']]['statut'] = 'active';
        }
        if (!isset($return[$type_widget['id']]['tarteaucitron'])){
            $return[$type_widget['id']]['tarteaucitron'] = 'non';
        }
        if (!isset($return[$type_widget['id']]['largeur_non_modifiable'])){
            $return[$type_widget['id']]['largeur_non_modifiable'] = 'non';
        }
        if (!isset($return[$type_widget['id']]['hauteur_non_modifiable'])){
            $return[$type_widget['id']]['hauteur_non_modifiable'] = 'non';
        }
        if (!isset($return[$type_widget['id']]['largeur_defaut'])){
            $return[$type_widget['id']]['largeur_defaut'] = '';
        }
        if (!isset($return[$type_widget['id']]['hauteur_defaut'])){
            $return[$type_widget['id']]['hauteur_defaut'] = '';
        }
    }
    
    $return['statut'] = $tableau_statut;
    $return['rgpd'] = $tableau_rgpd;
    
    return $return;    
}

// parametrage par le fichier _config_ciwidget.php dans /config
// ou parametrage par le fichier _config_ciwidget.php dans le dossier du plugin (a sa racine)
// Si les deux fichiers sont presents, celui present dans /config est prioritaire car il est propre au site
function ciwidget_charger_fichier_parametrage(){
    static $done = false;

    if (!$done){
        if (!defined('_CIWIDGET_CONFIG')){
            // parametrage par fichier
            $f = _DIR_ETC . '_config_ciwidget.php';
            // dans l'espace prive _DIR_PLUGIN_CIWIDGET commence par ../
            // car c'est _DIR_PLUGINS.'ciwidget/ et que _DIR_PLUGINS c'est _DIR_RACINE . "plugins/"
            $f2 = _DIR_PLUGIN_CIWIDGET . '_config_ciwidget.php';
            if (file_exists($f)) {
                    include_once($f);
            } elseif (file_exists($f2)) {
                    include_once($f2);
            }
        }
        $done = true;
    }

    return true;
}

function ciwidget_contenu_du_parametrage_par_fichier($arg=''){
    static $done = false;
    static $ciwidget_parametrage = array();

    if (!$done){
        // parametrage par fichier
        ciwidget_charger_fichier_parametrage();
        
        if (defined('_CIWIDGET_CONFIG')){
            $ciwidget_parametrage = unserialize(base64_decode(_CIWIDGET_CONFIG));
        }
        
        $done = true;
    }

    return $ciwidget_parametrage;
}

function ciwidget_contenu_du_parametrage_rgpd_par_fichier($arg=''){
    static $done = false;
    static $ciwidget_parametrage_rgpd = array();

    if (!$done){
        // parametrage par fichier
        ciwidget_charger_fichier_parametrage();
        
        if (defined('_CIWIDGET_RGPD')){
            $ciwidget_parametrage_rgpd = unserialize(base64_decode(_CIWIDGET_RGPD));
        }
        
        $done = true;
    }

    return $ciwidget_parametrage_rgpd;
}


// utilise par CISTYLE
function ciwidget_contenu_du_parametrage_par_constante($valeur=''){
    $return = array();

    $valeurs_possibles = array(
            'couleur',
            'palette',
            'styles'
            );
    
    // parametrage par fichier
    ciwidget_charger_fichier_parametrage();

    if (in_array($valeur,$valeurs_possibles)){
        $constante = '_CIWIDGET_'.strtoupper($valeur);
        $constante_cistyle = '_CISTYLE_'.strtoupper($valeur);
        if (defined($constante)){
            $return = unserialize(base64_decode(constant($constante)));
        } elseif (defined($constante_cistyle)){
            $return = unserialize(base64_decode(constant($constante_cistyle)));
        } 
    }
    
    return $return;
}

function ciwidget_parametrage_par_fichier($arg=''){
    static $done = false;
    static $ciwidget_parametrage = false;

    if (!$done){
        // parametrage par fichier
        ciwidget_charger_fichier_parametrage();
        
        if (defined('_CIWIDGET_CONFIG')){
            $ciwidget_parametrage = true;
        }
        
        $done = true;
    }

    return $ciwidget_parametrage;
}

function ciwidget_parametrage_couleur_par_fichier($arg=''){
    static $done = false;
    static $ciwidget_parametrage_couleur = false;

    if (!$done){
        // parametrage par fichier
        ciwidget_charger_fichier_parametrage();
        
        if (defined('_CIWIDGET_PALETTE') OR defined('_CIWIDGET_COULEUR')){
            $ciwidget_parametrage_couleur = true;
        }
        
        $done = true;
    }

    return $ciwidget_parametrage_couleur;
}

function ciwidget_parametrage_styles_par_fichier($arg=''){
    static $done = false;
    static $ciwidget_parametrage_styles = false;

    if (!$done){
        // parametrage par fichier
        ciwidget_charger_fichier_parametrage();
        
        if (defined('_CIWIDGET_STYLES')){
            $ciwidget_parametrage_styles = true;
        }
        
        $done = true;
    }

    return $ciwidget_parametrage_styles;
}

function ciwidget_parametrage_rgpd_par_fichier($arg=''){
    static $done = false;
    static $ciwidget_parametrage_rgpd = false;

    if (!$done){
        $ciwidget_parametrage_rgpd_par_fichier = ciwidget_contenu_du_parametrage_rgpd_par_fichier();
        if ($ciwidget_parametrage_rgpd_par_fichier){
            $ciwidget_parametrage_rgpd = true;
        }
    
        $done = true;
    }

    return $ciwidget_parametrage_rgpd;
}

function ciwidget_parametrage_widget_par_fichier($type_widget=''){
    $return = '';
    
    if ($type_widget){
        $ciwidget_contenu_parametrage_par_fichier = ciwidget_contenu_du_parametrage_par_fichier();
        if (isset($ciwidget_contenu_parametrage_par_fichier[$type_widget]) AND $ciwidget_contenu_parametrage_par_fichier[$type_widget]){
            $return = 'oui';
        }
    }
    
    if ($return){
        include_spip('inc/ciwidget_commun');    
        $types_widget = ciwidget_types_widget();

        foreach ($types_widget AS $cle=>$tableau_widget) {
            if (isset($tableau_widget['id']) AND $tableau_widget['id']==$type_widget) {
                // Gerer les exceptions non imposables
                if (isset($tableau_widget['non_imposable']) AND $tableau_widget['non_imposable']=='oui'){
                    $return = '';
                }
            }
        }
    }

    return $return;
}

function ciwidget_parametrage_rgpd_widget_par_fichier($type_widget=''){
    $return = '';
    
    if ($type_widget){
        $ciwidget_parametrage_rgpd_par_fichier = ciwidget_contenu_du_parametrage_rgpd_par_fichier();
        if (isset($ciwidget_parametrage_rgpd_par_fichier['rgpd']['rgpd_'.$type_widget]) AND $ciwidget_parametrage_rgpd_par_fichier['rgpd']['rgpd_'.$type_widget]=='oui'){
            $return = 'oui';
        }
    }
    
    return $return;
}

function ciwidget_choisir_statut($type_widget='',$valeur_element='',$disabled=''){
    $return = '';
    $selected = '';
    $disabled_texte = '';

    // tableau des valeurs
    $tableau_valeurs = array('active','masque','desactive');
    
    if (!$valeur_element){
        $valeur_element = 'active';
    }
    
    if (defined('_CIWIDGET_DESACTIVE_TOUS_WIDGETS_RGPD') AND _CIWIDGET_DESACTIVE_TOUS_WIDGETS_RGPD=='oui'){
        // desactivation forcée par constante des widgets concernés par le RGPD
        include_spip('inc/ciwidget_commun');
        $types_widget = ciwidget_types_widget();

        foreach ($types_widget AS $cle=>$valeur) {
            if (isset($valeur['id']) AND $valeur['id']==$type_widget){ 
                if (isset($valeur['concerne_rgpd']) AND $valeur['concerne_rgpd']=='oui'){
                    $disabled = true;
                    $valeur_element = 'desactive';
                    $tableau_valeurs = array('desactive');
                }
                break;
            }
        }
    }

    if ($disabled){
        $disabled_texte = 'disabled="disabled"';
    }
    
    // Parcours du tableau
    $return .= '<select name="statut" id="'.$type_widget.'_statut" '.$disabled_texte.'>';
    foreach($tableau_valeurs as $valeur){
        
        if($valeur == $valeur_element){
            $selected = ' selected="selected"';
        } else {
            $selected = '';
        }
        
        // Affichage de la ligne
        $return .= "\t".'<option value="'. $valeur .'"'. $selected . '>'. _T('ciwidget:statut_'.$valeur) .'</option>'."\n";
    }
    $return .= '</select>'."\n";

    return $return;
}

function ciwidget_choisir_statut_widget($type_widget='',$valeur_element=''){
    $return = '';
    $selected = '';
    $disabled = '';
    $disabled_texte = '';

    // tableau des valeurs
    $tableau_valeurs = array('active','masque','desactive');
    
    if (!$valeur_element){
        $valeur_element = 'active';
    }
    
    if (defined('_CIWIDGET_DESACTIVE_TOUS_WIDGETS_RGPD') AND _CIWIDGET_DESACTIVE_TOUS_WIDGETS_RGPD=='oui'){
        // desactivation forcée par constante des widgets concernés par le RGPD
        include_spip('inc/ciwidget_commun');
        $types_widget = ciwidget_types_widget();

        foreach ($types_widget AS $cle=>$valeur) {
            if (isset($valeur['id']) AND $valeur['id']==$type_widget){ 
                if (isset($valeur['concerne_rgpd']) AND $valeur['concerne_rgpd']=='oui'){
                    $disabled = true;
                    $valeur_element = 'desactive';
                    $tableau_valeurs = array('desactive');
                }
                break;
            }
        }
    }

    if ($disabled OR base64_encode($type_widget)=='Z3JhcGhpcXVl'){
        $disabled_texte = 'disabled="disabled"';
    }
    
    // Parcours du tableau
    $return .= '<select name="statut_'.$type_widget.'" id="'.$type_widget.'" '.$disabled_texte.'>';
    foreach($tableau_valeurs as $valeur){
        
        if($valeur == $valeur_element){
            $selected = ' selected="selected"';
        } else {
            $selected = '';
        }
        
        // Affichage de la ligne
        $return .= "\t".'<option value="'. $valeur .'"'. $selected . '>'. _T('ciwidget:statut_'.$valeur) .'</option>'."\n";
    }
    $return .= '</select>'."\n";

    return $return;
}

function ciwidget_choisir_video_num_skin($nom_element='',$valeur_element='',$disabled=''){
    $return = '';
    $selected = '';
    $disabled_texte = '';
    if ($disabled){
        $disabled_texte = 'disabled="disabled"';
    }
    if (!$valeur_element){
        $valeur_element = 'active';
    }
    
    // tableau des valeurs
    $tableau_valeurs = array('1','2','3');
    // Parcours du tableau
    $return .= '<select name="'.$nom_element.'" '.$disabled_texte.'>';
    foreach($tableau_valeurs as $valeur){
        
        if($valeur == $valeur_element){
            $selected = ' selected="selected"';
        } else {
            $selected = '';
        }
        
        // Affichage de la ligne
        $return .= "\t".'<option value="'. $valeur .'"'. $selected . '>'. _T('ciwidget:video_num_skin_'.$valeur) .'</option>'."\n";
    }
    $return .= '</select>'."\n";

    return $return;
}

function formulaires_ciwidget_configurer_charger($widget='') {
    
    if (!autoriser('configurer configuration'))
        return false;

    $valeurs = ciwidget_lire_meta_widget($widget);

    return $valeurs;
}

function formulaires_ciwidget_configurer_verifier($widget='') {
    $erreurs = array();
    
    if (_request('reset')) {
        return $erreurs;
    }
    
    include_spip('inc/ciwidget_inc_style');

    $tableau_des_champs = array();
    $ciwidget_defaut = ciwidget_valeurs_par_defaut();
    if (isset($ciwidget_defaut[$widget])) {
        $tableau_des_champs = $ciwidget_defaut[$widget];
    }
    
    foreach ($tableau_des_champs AS $nom_champ=>$valeur_defaut){
        if (isset($_POST[$nom_champ])){
            if (_request($nom_champ)){
                if (!ciwidget_verifier($nom_champ,_request($nom_champ))){
                    $erreurs[$nom_champ] =  _T('ciwidget:valeur_incorrecte');
                }
            }
        }
    }
    
    return $erreurs;
}

function formulaires_ciwidget_configurer_traiter($widget='') {
    $res = array();
    $valeurs_actuelles = ciwidget_lire_meta_widget($widget);
    $valeurs_futures = array();
    
    if (_request('reset')) {
        $res['redirect'] = generer_url_ecrire("configurer_ciwidget");
        refuser_traiter_formulaire_ajax();
    }
    
    $tableau_des_champs = array();
    $ciwidget_defaut = ciwidget_valeurs_par_defaut();
    if (isset($ciwidget_defaut[$widget])) {
        $tableau_des_champs = $ciwidget_defaut[$widget];
    }

    if (_request('reset')) {
        ciwidget_supprime_meta_widget($widget);
        
    } else {
        foreach ($tableau_des_champs AS $nom_champ=>$valeur_defaut){
            if (isset($_POST[$nom_champ])){
                $valeurs_futures[$nom_champ] = _request($nom_champ);
            }    
        }
        ciwidget_ecrire_meta($widget,$valeurs_futures);
    }
    
    $res['message_ok'] = _T('config_info_enregistree');
    $res['redirect'] = generer_url_ecrire("configurer_ciwidget");
    
    return $res;
}

?>