<?php
/**
 * Plugin CIWIDGET
 * Copyright (c) Christophe IMBERTI
 * Licence Creative commons by-nc-sa
 */

/**
 * Analyse des textes pour trouver et marquer comme vu les widgets utilisés dedans
 **/

if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

// la dist ne regarde que chapo et texte, on laisse comme ca,
// mais ca permet d etendre a descriptif ou toto depuis d autres plugins
$GLOBALS['ciwidget_liste_champs'] = array();
$GLOBALS['ciwidget_liste_champs'][] = 'texte';
$GLOBALS['ciwidget_liste_champs'][] = 'descriptif';

/**
 * Trouver les widgets utilisés dans le texte d'un objet et enregistrer cette liaison comme vue.
 *
 * La liste des champs susceptibles de contenir des widgets ou images est indiquée
 * par la globale `ciwidget_liste_champs` (un tableau).
 *
 * Le contenu de ces champs (du moins ceux qui existent pour l'objet demandé) est récupéré et analysé.
 * La présence d'un modèle de widget dans ces contenus
 * indique que le widget est utilisé et doit être lié à l'objet, avec le champ `vu=oui`
 *
 * S'il y avait des anciens liens avec vu=oui qui n'ont plus lieu d'être, ils passent à non.
 *
 * @param array $champs
 *     Couples [champ => valeur] connus de l'objet
 * @param int $id
 *     Identifiant de l'objet
 * @param string $type
 *     Type d'objet éditorial (ex: article)
 * @param string $id_table_objet
 *     Nom de la clé primaire sur la table sql de l'objet
 * @param string $table_objet
 *     Nom de l'objet éditorial (ex: articles)
 * @param string $spip_table_objet
 *     Nom de la table sql de l'objet
 * @param array $desc
 *     Description de l'objet, si déjà calculé
 * @param string $serveur
 *     Serveur sql utilisé.
 * @return void|null
 **/
function inc_marquer_doublons_widget_dist(
	$champs,
	$id,
	$type,
	$id_table_objet,
	$table_objet,
	$spip_table_objet,
	$desc = array(),
	$serveur = ''
) {
	if (!$champs) {
		return;
	}
        
	// On conserve uniquement les champs qui modifient le calcul des doublons de widgets
	// S'il n'y en a aucun, les doublons ne sont pas impactés, donc rien à faire d'autre..
	if (!$champs = array_intersect_key($champs, array_flip($GLOBALS['ciwidget_liste_champs']))) {
		return;
	}

	if (!$desc) {
		$trouver_table = charger_fonction('trouver_table', 'base');
		$desc = $trouver_table($table_objet, $serveur);
	}

	// Il faut récupérer toutes les données qui impactent les liens de widgets vus
	// afin de savoir lesquels sont présents dans les textes, et pouvoir actualiser avec
	// les liens actuellement enregistrés.
	$absents = array();

	// Récupérer chaque champ impactant qui existe dans la table de l'objet et qui nous manque
	foreach ($GLOBALS['ciwidget_liste_champs'] as $champ) {
		if (isset($desc['field'][$champ]) and !isset($champs[$champ])) {
			$absents[] = $champ;
		}
	}

	// Retrouver les textes des champs manquants
	if ($absents) {
		$row = sql_fetsel($absents, $spip_table_objet, "$id_table_objet=" . sql_quote($id));
		if ($row) {
			$champs = array_merge($row, $champs);
		}
	}

	include_spip('inc/texte');
	include_spip('base/abstract_sql');
	include_spip('action/editer_liens');
	include_spip('base/objets');
        include_spip('inc/ciwidget_commun');    

	// récupérer la liste des modèles qui considèrent un widget comme vu s'ils sont utilisés dans un texte
	$modeles = ciwidget_tableau_id_type_widget();

	// liste d'id_widget trouvés dans les textes
        $texte_widgets_vus = array();

        
        if (!defined('_CIWIDGET_RACCOURCI_MODELE')){
            define('_CIWIDGET_RACCOURCI_MODELE',
                    '(<widget_([a-z_-]+)' # <modele
                    . '\s*([0-9]*)\s*' # id
                    . '(|\|center|\|left|\|right)' # |arguments 
                    . '>)' # fin du modele >
            );
        }
        
       	// detecter les modeles (rapide)
        $texte = implode(' ', $champs);
	if (strpos($texte, "<") !== false and
		preg_match_all('/<widget_[a-z_-]+\s*[0-9|]+/iS', $texte, $matches, PREG_SET_ORDER)
	) {
		foreach ($matches as $match) {
			// Recuperer l'appel complet (y compris un eventuel lien)
			$a = strpos($texte, $match[0]);
                        preg_match('@^' ._CIWIDGET_RACCOURCI_MODELE. '@isS', substr($texte, $a), $regs);
                        if (isset($regs[2]) AND in_array($regs[2],$modeles)){
                            $texte_widgets_vus[] = $regs[3]; 
                        }
                }
        }
        
        
	// on ne modifie les liaisons que si c'est nécessaire
	$bdd_widgets_vus = array(
		'oui' => array(),
		'non' => array()
	);

        $result = sql_select("id_widget,vu", "spip_ciwidget", "objet='".$type."' AND id_objet=".intval($id));
        while ($l = sql_fetch($result)) { 
		$bdd_widgets_vus[$l['vu']][] = $l['id_widget'];
        }

	// il y a des nouveaux widgets vus dans le texte
	$nouveaux = array_diff($texte_widgets_vus, $bdd_widgets_vus['oui']);

	// il y a des anciens widgets vus dans la bdd
	$anciens = array_diff($bdd_widgets_vus['oui'], $texte_widgets_vus);

	if ($nouveaux) {
		// on vérifie que les widgets indiqués vus existent réellement tout de même (en cas d'erreur de saisie)
		$rows = sql_allfetsel('id_widget', 'spip_ciwidget', sql_in('id_widget', $nouveaux));
                $ids = array();
        	foreach($rows as $row) {
                    $ids[] = $row['id_widget'];
                }
		if ($ids) {
                        sql_updateq("spip_ciwidget", array('vu' => 'oui'), sql_in('id_widget',$ids));
		}
	}

	if ($anciens) {
                sql_updateq("spip_ciwidget", array('vu' => 'non'), sql_in('id_widget',$anciens));
	}
}

?>